# This function takes a clean extrapolated airfoil coefficient file and produces
# an AeroDyn formated file. 
#
# Inputs:
#   - input_file: (string) template file
#   - output_file: (string) name of the file to create 
#   - polor_file: (string) file of clean extrapolated airfoil polor
#   - RE: (float) Reynolds number polor calculated on
#   - plot: (bool) True or False whether to show CL/CD/CN plot

def input_builder(input_file, output_file, polor_file, RE, plot):
    
    import pandas as pd
    import numpy as np
    #-------------------------------------------------------------------------#
    
    
    # import polor
    polor = pd.read_csv(polor_file, sep='\t', lineterminator='\n', 
                        usecols=range(0,5), header=None, 
                        names = ['alpha', 'CL', 'CD', 'CM', 'CDp'])
    
    polor.iloc[0,4] = polor.iloc[-1,4] # a fix to stop inconsistencies in AeroDyn
    
    polor['CN'] = pd.Series(np.zeros(len(polor)), index=polor.index)
    polor['CT'] = pd.Series(np.zeros(len(polor)), index=polor.index)
    
    for i in range(len(polor)):
        polor.CN.loc[i] = polor.CL.loc[i] * np.cos(polor.alpha.loc[i]*np.pi/180) + polor.CD.loc[i] * np.sin(polor.alpha.loc[i]*np.pi/180)
        polor.CT.loc[i] = - polor.CL.loc[i] * np.sin(polor.alpha.loc[i]*np.pi/180) + polor.CD.loc[i] * np.cos(polor.alpha.loc[i]*np.pi/180)
    

    choice = str(input("Do you want to manually input CN slope points? y/n: "))
    while choice not in ('y', 'n'):
        choice = str(input("Dyo you want to manually input CN slope points? y/n: "))
    if choice == 'y':
        p1 = int(input("CN slope p1 index: "))
        while p1 not in polor.index:
            p1 = int(input("The index selected for p1 is not in polor. Please select CN slope p1 index: "))
        p2 = int(input("CN slope p2 index: "))
        while p2 not in polor.index:  
            p2 = int(input("The index selected for p2 not in polor. Please select CN slope p2 index: "))       
    else:
        pass
    #-------------------------------------------------------------------------#
     
    
    # Find zero lift and interp other values at zero lift
    CL  = np.array(polor.CL)
    z_c = np.where(np.diff(np.sign(CL)))[0]
        
    for i in z_c:
        if polor.iloc[i,0] < 60 and polor.iloc[i,0] > -60:
            z_c = i
            
    CL0_subset = np.array(polor.iloc[z_c:z_c+2,:])
    
    AoA0 = np.interp(0, CL0_subset[:,1], CL0_subset[:,0]) 
    CD0  = np.interp(0, CL0_subset[:,1], CL0_subset[:,2])
    CM0  = np.interp(0, CL0_subset[:,1], CL0_subset[:,3])
    #-------------------------------------------------------------------------#
    
    
    # Find stall angles +ve & -ve
    if (polor.alpha.isin([0]).any()) == True:
        Zero_AoA = polor.alpha[polor.alpha == 0].index.tolist()
        Zero_AoA = Zero_AoA[0]
        CL_p = polor.CL.loc[Zero_AoA:]
        CL_n = polor.CL.loc[0:Zero_AoA]
    else:
        AoA = np.array(polor.alpha)
        z_c = int(np.where(np.diff(np.sign(AoA)))[0])
        CL_p = polor.CL.loc[z_c+1:]
        CL_n = polor.CL.loc[0:z_c]
    
    Stall_Ang_p = polor.alpha.loc[CL_p.idxmax()]
    Stall_Ang_n = polor.alpha.loc[CL_n.idxmin()]
    
    print('+ve stall angle: '+str(Stall_Ang_p))
    print('-ve stall angle: '+str(Stall_Ang_n))
    #-------------------------------------------------------------------------#
    
    
    # Find normal force coeffs (CN) at max lift (AoA<0 & AoA>0)
    if (polor.alpha.isin([Stall_Ang_p]).any()) == True:
        ind = polor.index[polor['alpha'] == Stall_Ang_p].tolist()[0]
        CN_p = polor.CN.loc[ind]
    else:
        ind = polor.index[polor['alpha'] == 0].tolist()[0]
        CN_p_sub = np.array(polor.iloc[ind:,:])
        CN_p  = np.interp(Stall_Ang_p, CN_p_sub[:,0], CN_p_sub[:,5])
    
    if (polor.alpha.isin([Stall_Ang_n]).any()) == True:
        ind = polor.index[polor['alpha'] == Stall_Ang_n].tolist()[0]
        CN_n = polor.CN.loc[ind]
    else:
        ind = polor.index[polor['alpha'] == 0].tolist()[0]
        CN_n_sub = np.array(polor.iloc[:ind,:])
        CN_n  = np.interp(Stall_Ang_n, CN_n_sub[:,0], CN_n_sub[:,5])
    #-------------------------------------------------------------------------#
        
        
    # Find slope of Normal Force Coeff (C_nalpha)
    AoA = np.array(polor.alpha)
    CN  = np.array(polor.CN)
    
    grad  = np.gradient(CN,AoA)
    dgrad = np.gradient(grad,AoA)
       
    AoA0_ind = int(np.where(AoA == 0)[0])
    
    if choice == 'n':
        p1 = int(np.argwhere(abs(dgrad) == max(abs(dgrad[:AoA0_ind]))))  
        z  = np.argwhere(np.diff(np.sign(grad))!= 0)
        z  = z[z>AoA0_ind]
        p2 = z[0]
       
    AoA_s1 = AoA[p1] * np.pi/180
    AoA_s2 = AoA[p2] * np.pi/180
    CN_s1  = CN[p1]
    CN_s2  = CN[p2]

    CN_slope = ( CN_s2 - CN_s1 ) / ( AoA_s2 - AoA_s1 )
    #-------------------------------------------------------------------------#
    
    
    # Insert new inputs to airfoil file
    table_start = 999999
    str_polor = polor[['alpha', 'CL', 'CD', 'CM', 'CDp']].to_string(header=False, index=False, index_names=False).split('\n')
    
    with open(input_file, 'r') as fin, open(output_file, 'w') as fout:
        
        for i, line in enumerate(fin):
            
            lineout = line
            
            if '! Reynolds number in millions' in line:
                lineout = str(RE) + " \t\t\t\t\tRe\t\t\t\t\t\t\t\t! Reynolds number in millions\n"
            if '! 0-lift angle of attack, depends on airfoil.' in line:
                lineout = str(round(AoA0,3)) + " \t\t\t\talpha0\t\t\t\t\t\t! 0-lift angle of attack, depends on airfoil.\n"        
            if '! Angle of attack at f=0.7, (approximately the stall angle) for AOA>alpha0. (deg)' in line:
                lineout = str(Stall_Ang_p) + " \t\t\t\t\talpha1\t\t\t\t\t\t! Angle of attack at f=0.7, (approximately the stall angle) for AOA>alpha0. (deg)\n"        
            if '! Angle of attack at f=0.7, (approximately the stall angle) for AOA<alpha0. (deg)' in line:
                lineout = str(Stall_Ang_n) + " \t\t\t\talpha2\t\t\t\t\t\t! Angle of attack at f=0.7, (approximately the stall angle) for AOA<alpha0. (deg)\n"                 
            if '! Slope of the 2D normal force coefficient curve. (1/rad)' in line:
                lineout = str(round(CN_slope,4)) + "\t\t\t\tC_nalpha\t\t\t\t\t! Slope of the 2D normal force coefficient curve. (1/rad)\n"            
            if '! Critical value of C0n at leading edge separation.' in line:
                lineout = str(round(CN_p,3)) + " \t\t\t\tCn1\t\t\t\t\t\t\t\t! Critical value of C0n at leading edge separation. It should be extracted from airfoil data at a given Mach and Reynolds number. It can be calculated from the static value of Cn at either the break in the pitching moment or the loss of chord force at the onset of stall. It is close to the condition of maximum lift of the airfoil at low Mach numbers.\n"        
            if '! As Cn1 for negative AOAs.' in line:
                lineout = str(round(CN_n,3)) + " \t\t\t\tCn2\t\t\t\t\t\t\t\t! As Cn1 for negative AOAs.\n"
            if '! 2D drag coefficient value at 0-lift.' in line:
                lineout = str(round(CD0,4)) + " \t\t\t\tCd0\t\t\t\t\t\t\t\t! 2D drag coefficient value at 0-lift.\n"                
            if '! 2D pitching moment coefficient about 1/4-chord location, at 0-lift, positive if nose up.' in line:
                lineout = str(round(CM0,4)) + " \t\t\tCm0\t\t\t\t\t\t\t\t! 2D pitching moment coefficient about 1/4-chord location, at 0-lift, positive if nose up. [If the aerodynamics coefficients table does…\n"        
            if '! Number of data lines in the following table' in line:
                lineout = str(len(polor)) + " \t\t\t\t\tNumAlf\t\t\t\t\t\t! Number of data lines in the following table\n"
            if '!    (deg)      (-)     (-)   (-)   (-)' in line:
                table_start  = i
            if i > table_start:
                lineout = ''
            
            fout.write(lineout) 
    
    with open(output_file, 'a') as fout:
        for i in str_polor:
            lineout = i + '\n'
            fout.write(lineout)
    #-------------------------------------------------------------------------#
    
    
    # plot CL/CD curves if user specified
    if plot == True:
        import matplotlib.pyplot as plt
        fig, ax = plt.subplots()
        ax.plot(polor.iloc[:,0],polor.iloc[:,1])
        ax.plot(polor.iloc[:,0],polor.iloc[:,2])
        ax.plot(polor.iloc[:,0],polor.iloc[:,5])
        ax.plot([AoA_s1*(180/np.pi),AoA_s2*(180/np.pi)],[CN_s1,CN_s2],linestyle='--')
        ax.spines['left'].set_position('zero')
        ax.spines['right'].set_color('none')
        ax.yaxis.tick_left()
        ax.spines['bottom'].set_position('zero')
        ax.spines['top'].set_color('none')
        ax.xaxis.tick_bottom()
        ax.grid(linestyle='--')
        ax.legend(['CL', 'CD', 'CN', 'CN_slope'])
        ax.set_xlim([-195, 195])
        ax.set_ylim([-1.6, 1.6])
        plt.xticks(np.arange(-180,185,60))
        #---------------------------------------------------------------------#    
    
    
    return polor


   
 

input_file  = 'template.dat'
polor_file  = '../XFOIL_Polors/Xfoil_Outputs/Extrap_Polors/Extrap_PolorRe0_01e6.dat'
output_file = 'NACA_63418_Re0_01.dat'
RE = 0.15
plot = True
polor = input_builder(input_file, output_file, polor_file, RE, plot)